@@ -1,18 +1,6 @@
 #####################################
 ######priliminary utilities##########
 #####################################
-bivcenindic <- function(Cen1,Cen2){
-ar<-array(1:length(Cen1))
-ar1=Cen1*Cen2*ar
-ar11<-ar1[ar1>0]
-ar1=Cen1*(1-Cen2)*ar
-ar10<-ar1[ar1>0]
-ar1=(1-Cen1)*Cen2*ar
-ar01<-ar1[ar1>0]
-ar1=(1-Cen1)*(1-Cen2)*ar
-ar00<-ar1[ar1>0]
-list(Ic11=ar11,Ic10=ar10,Ic01=ar01,Ic00=ar00)}
-#
 clusievent <- function(x,fx){
 ar<-array(x[1]:x[2])
 kr=sum(fx[ar])
@@ -28,9 +16,11 @@ ar<-array(x[1]:x[2])
 kr=fx[ar[1]]
 kr}
 #
+# to obtain 95% CI coverage
 coverp <- function(x,L,R) ifelse(x>=L & x<=R,1,0)
 zq=abs(qnorm(0.025,mean=0,sd=1))# Z alpha/2
 #
+#
 # log gamma function i.e., log(gamma(1/a+x)) for x=0,1,2,..
 funloggamm <- function(x,a){
 kr=sum(log(1/a+array(0:x)))-log(1/a+x)
@@ -78,14 +68,6 @@ macl=matrix(c(macl),nr,nc)
 kr=colSums(macl)
 kr}
 #
-newmoppp <- function(yy,daDI,drDI){
-WI=matrix(c(yy[1],yy[2],yy[2],yy[3]),2,2)
-HIda=(((WI%*%daDI%*%WI)%*%daDI))
-HIdar=(((WI%*%drDI%*%WI)%*%daDI))
-HIdr=(((t(WI%*%drDI)%*%(WI%*%drDI))))
-c(0.5*sum(diag(HIda)),0.5*sum(diag(HIdar)),0.5*sum(diag(HIdr)))
-}
-#
 funfittypp <- function(clustind){
 Tfr=table(clustind);Tfr=as.matrix(Tfr)
 tfr=as.vector(Tfr[,1])
@@ -119,21 +101,24 @@ if(length(cdeb)==0){re=c(0)}
 if(length(cdeb)>0){re=c(sum(cen[cdeb]))}
 re}
 #
+bivcenindic <- function(Cen1,Cen2){
+ar<-array(1:length(Cen1))
+ar1=Cen1*Cen2*ar
+ar11<-ar1[ar1>0]
+ar1=Cen1*(1-Cen2)*ar
+ar10<-ar1[ar1>0]
+ar1=(1-Cen1)*Cen2*ar
+ar01<-ar1[ar1>0]
+ar1=(1-Cen1)*(1-Cen2)*ar
+ar00<-ar1[ar1>0]
+list(Ic11=ar11,Ic10=ar10,Ic01=ar01,Ic00=ar00)}
+#
 # to generate bivariate normal random effect
 rbivnorm<-function(psize,ssq,r){
 w1=rnorm(psize,mean=0,sd=sqrt(ssq))
 w2=rnorm(psize,mean=r*w1,sd=sqrt((1-r^2)*ssq))
 cbind(w1,w2)}
-#
-#
-weigtevent <- function(uniind,x,we,cen){
-cde=ifelse(uniind[1]==x,1,0)
-cdr=c(cen)*cde*array(1:length(x))
-cdeb=as.vector(cdr[cdr>0])
-if(length(cdeb)==0){re=c(0)}
-if(length(cdeb)>0){re=c(sum(we[cdeb]))}
-re}
-#
+
 sortingfunc <- function(time,censor,weights){
 order=sort(time, decreasing = FALSE, index.return = TRUE);indx=order$ix;timeo=order$x
 uniq_tim<-unique(sort(time))
@@ -164,6 +149,14 @@ n_evep[n_evep<=0]<-1
 list(uniq_tim=uniq_tim,ind.haz=ind.haz,n_eve=n_eve,n_evep=n_evep,
 t_n_eve=t_n_eve,eventtau=eventtau,weights=weights,indx=indx,t=time)}
 #
+weigtevent <- function(uniind,x,we,cen){
+cde=ifelse(uniind[1]==x,1,0)
+cdr=c(cen)*cde*array(1:length(x))
+cdeb=as.vector(cdr[cdr>0])
+if(length(cdeb)==0){re=c(0)}
+if(length(cdeb)>0){re=c(sum(we[cdeb]))}
+re}
+#
 # for derivatives in censoring cases (1,1),(1,0), and (0,1)
 #
 d2barderivP11cv= function (Ic11,newtht,Y1,Y2,H1,H2){
@@ -740,6 +733,89 @@ Hes=colSums(sr1$hessian)
 }
 #
 
+
+.Llikgammacvy = function (theta,X,Y,RI,H_bet_x,Othrs,ICs){
+censor=Othrs$censor;ind.haz=Othrs$ind.haz;n_eve=Othrs$n_eve
+i1=ICs$i1;i2=ICs$i2
+HHD=Z=W=H_bet_x
+a = abs(theta[1]);R = abs(theta[2])
+cen1<-cen2<-newdiff <-beto<-NULL
+cen1=censor[i1];cen2=censor[i2]
+bet<-rep(0,ncol(X))
+iter1=0
+repeat{
+iter1=iter1+1
+beto=bet
+e_frail<-.Expefrail(newtht=theta,H1=HHD[i1],H2=HHD[i2],cen1=cen1,cen2=cen2)
+Z[i1]<-e_frail$z1;Z[i2]<-e_frail$z2;W<-log(Z);W[is.na(W)]<-0
+cph1 <- survival::coxph.fit(x = X, y = Y, strata = NULL,
+offset =W, init = bet, control = survival::coxph.control(),
+method = "breslow", resid=FALSE)
+bet<-cph1$coefficients
+x_bet<-X%*%bet
+svexp_bet_xo=as.vector(RI%*%(exp(x_bet+W)))
+H0<-cumsum(c(n_eve/svexp_bet_xo))
+HHD=c(H0[ind.haz]*exp(c(x_bet)))
+newdiff=max(abs(bet-beto))
+if((newdiff <= 1e-06)  |  (iter1 >= 20)) break}
+H1=HHD[i1];H2=HHD[i2]
+HH=(H1+H2)
+QQ=(cph1$loglik[2]-sum(censor*W)+sum(censor))
+logli=QQ+sum((-((1-R)/a)*log(1+a*H1)-((1-R)/a)*log(1+a*H2)-(R/a)*log(1+a*(HH)))+
+cen1*cen2*log((R+a)*R*(1+a*(HH))^(-2)+(1-R)*R*((1+a*(HH))^(-1))*((1+a*H1)^(-1)+(1+a*H2)^(-1))+
+((1-R)^2)*((1+a*H1)^(-1))*((1+a*H2)^(-1)))+
+cen1*(1-cen2)*log(R*((1+a*(HH))^(-1))+(1-R)*((1+a*H1)^(-1)))+
+(1-cen1)*cen2*log(R*((1+a*(HH))^(-1))+(1-R)*((1+a*H2)^(-1))))
+-logli
+    }
+#
+.fdLlikgammacvy = function (theta,X,Y,RI,H_bet_x,Othrs,ICs){
+censor=Othrs$censor;ind.haz=Othrs$ind.haz;n_eve=Othrs$n_eve
+i1=ICs$i1;i2=ICs$i2
+HHD=Z=W=H_bet_x
+a = abs(theta[1]);R = abs(theta[2])
+cen1<-cen2<-newdiff <-beto<-NULL
+cen1=censor[i1];cen2=censor[i2]
+bet<-rep(0,ncol(X))
+iter1=0
+repeat{
+iter1=iter1+1
+beto=bet
+e_frail<-.Expefrail(newtht=theta,H1=HHD[i1],H2=HHD[i2],cen1=cen1,cen2=cen2)
+Z[i1]<-e_frail$z1;Z[i2]<-e_frail$z2;W<-log(Z);W[is.na(W)]<-0
+cph1 <- survival::coxph.fit(x = X, y = Y, strata = NULL,
+offset =W, init = bet, control = survival::coxph.control(),
+method = "breslow", resid=FALSE)
+bet<-cph1$coefficients
+x_bet<-X%*%bet
+svexp_bet_xo=as.vector(RI%*%(exp(x_bet+W)))
+H0<-cumsum(c(n_eve/svexp_bet_xo))
+HHD=c(H0[ind.haz]*exp(c(x_bet)))
+newdiff=max(abs(bet-beto))
+if((newdiff <= 1e-06)  |  (iter1 >= 20)) break}
+H1=HHD[i1];H2=HHD[i2]
+da=sum(((1-R)/a^2*(log(1+a*H1) + log(1 + a * H2))-((1-R)/a) * (H1/(1 + a * H1) + H2/(1 + a * H2)) - ((R/a) * ((H1 + H2)/(1 +
+a *(H1+H2)))-R/a^2 * log(1 + a * (H1 +H2))) + cen1 * cen2 * ((R * (1 + a * (H1 + H2))^(-2) + (R + a) * R * ((1 +
+a *(H1+H2))^((-2) - 1) * ((-2) * (H1 +H2))) + ((1 - R) * R * ((1 + a * (H1 + H2))^((-1) - 1) * ((-1) * (H1 + H2))) *
+((1 + a * H1)^(-1) + (1 + a * H2)^(-1)) + (1-R) * R * (1+a * (H1 + H2))^(-1) * ((1 + a * H1)^((-1) - 1) * ((-1) *
+H1) + (1 + a * H2)^((-1) - 1) * ((-1) * H2))) + (((1 - R)^2) * ((1 + a * H1)^((-1) - 1) * ((-1) * H1)) * (1 + a * H2)^(-1) +
+((1 - R)^2) * (1 + a * H1)^(-1) * ((1 + a * H2)^((-1) - 1) * ((-1) * H2))))/((R + a) * R * (1 + a * (H1 + H2))^(-2) +
+(1 - R) * R * (1 + a * (H1 + H2))^(-1) * ((1 + a * H1)^(-1) + (1 + a * H2)^(-1)) + ((1 - R)^2) * (1 + a * H1)^(-1) *
+(1 + a * H2)^(-1))) + cen1 * (1 - cen2) * ((R * ((1 + a * (H1 + H2))^((-1) - 1) * ((-1) * (H1 + H2))) + (1 - R) * ((1 +
+a * H1)^((-1) - 1) * ((-1) * H1)))/(R * (1 + a * (H1 + H2))^(-1) + (1 - R) * (1 + a * H1)^(-1))) + (1 - cen1) * cen2 * ((R *
+((1 + a * (H1 + H2))^((-1) - 1)*((-1) * (H1 + H2))) + (1 - R) * ((1 + a * H2)^((-1) - 1) * ((-1) * H2)))/(R * (1 + a *
+(H1 + H2))^(-1) + (1 - R) * (1 + a * H2)^(-1)))))
+dR=sum((1/a * (log(1 + a * H1) + log(1 + a * H2))-1/a * log(1 + a * (H1 + H2)) + cen1 * cen2 * (((R + (R + a)) * (1 + a * (H1 +
+H2))^(-2) + ((1 - R) - R) * (1 + a * (H1 + H2))^(-1) * ((1 + a * H1)^(-1) + (1 + a * H2)^(-1)) - 2 * (1 - R) * (1 + a *
+H1)^(-1) * (1 + a * H2)^(-1))/((R + a) * R * (1 + a * (H1 + H2))^(-2) + (1 - R) * R * (1 + a * (H1 + H2))^(-1) * ((1 +
+a * H1)^(-1) + (1 + a * H2)^(-1)) + ((1 - R)^2) * (1 + a * H1)^(-1) * (1 + a * H2)^(-1))) + cen1 * (1 - cen2) * (((1 +
+a * (H1 + H2))^(-1) - (1 + a * H1)^(-1))/(R * (1 + a * (H1 + H2))^(-1) + (1 - R) * (1 + a * H1)^(-1))) + (1 - cen1) *
+cen2 * (((1 + a * (H1+ H2))^(-1) - (1 + a * H2)^(-1))/(R * (1 + a * (H1 + H2))^(-1) + (1 - R) * (1 + a * H2)^(-1)))))
+gd=c(da,dR)
+-gd
+}
+#
+
 # function for the case when sampling weight is included in gamma fit
 #
 .optimcvw<-function(newtht,xx,control){
@@ -778,23 +854,25 @@ datu=data.frame(time=time,censor=censor,X)
 IID=array(1:data.n1);ar=array(1:data.n);PID=rep(ar,each=2)
 form<-as.formula(paste(paste("Surv(time,censor)"),"~",paste(colnames(X), collapse = "+ "),
 "+ frailty(IID)",collapse=""))
-shrfit <- shrgamsp(form,data=datu,weights=weights)
-bet=shrfit$coefficients;x_bet<-X%*%bet;W=log(shrfit$frail)
+cph.fit <- coxph(form,data=datu,weights=weights,method = "breslow")
+bet=cph.fit$coefficients;x_bet<-X%*%bet;W=cph.fit$frail
 svexp_bet_xo=as.vector(RI%*%((c(weights)*exp(x_bet+W))))
 H0<-cumsum(c(n_eve/svexp_bet_xo))
 H_bet_x=c(H0[ind.haz]*exp(c(x_bet)))
-tht=shrfit$frailparest
+re=cph.fit$history$`frailty(IID)`$history;re1=re[,1];tht=re1[length(re1)]
 newtht=c(tht,0.5)
 if(tht<0.0001){
 form<-as.formula(paste(paste("Surv(time,censor)"),"~",paste(colnames(X), collapse = "+ "),
 "+ frailty(PID)",collapse=""))
-shrfit <- shrgamsp(form,data=datu,weights=weights)
-bet=shrfit$coefficients;x_bet<-X%*%bet;w=log(shrfit$frail)
+cph.fit <- coxph(form,data=datu,weights=weights,method = "breslow")
+bet=cph.fit$coefficients
+x_bet<-X%*%bet
+w=cph.fit$frail
 W[i1]<-w;W[i2]<-w
 svexp_bet_xo=as.vector(RI%*%((c(weights)*exp(x_bet+W))))
 H0<-cumsum(c(n_eve/svexp_bet_xo))
 H_bet_x=c(H0[ind.haz]*exp(c(x_bet)))
-tht=shrfit$frailparest
+re=cph.fit$history$`frailty(PID)`$history;re1=re[,1];tht=re1[length(re1)]
 newtht=c(tht,0.5)
 if(tht<0.0001){newtht=c(0.001,0.001)}}}
 iter=0
@@ -877,9 +955,383 @@ res
 ############### End Utilities for Bivariate gamma frailty fit #####
 ###################################################################
 #
-######################################################
-###########Univariate and shared gamma############
-#######################################
+###################################################################
+###############Utilities for Bivariate Log normal frailty##########
+###################################################################
+llpennlognGeni<-function (par,newtht,censor,X,RI,n_eve,ind.haz,i1,i2){
+ncoef=ncol(X);n1=nrow(X)
+a=newtht[1];roh=newtht[2]
+bet=par[1:ncoef]
+W=par[(ncoef+1):length(par)]
+W1=W[i1];W2=W[i2]
+x_bet<-X%*%bet
+x_bet_w<-X%*%bet+W
+expx_bet_w<-exp(c(x_bet_w))
+svexp_bet_xo=as.vector(RI%*%(exp(x_bet_w)))
+dbpart1svexp=(RI%*%(c(exp(x_bet_w))*X))
+af=c(n_eve/svexp_bet_xo)
+dbsvexp_bet_xo=as.vector(colSums(af*dbpart1svexp))
+sumx=as.vector(colSums((c(censor)*X)))
+H0<-cumsum(c(n_eve/svexp_bet_xo))
+H_bet_xw=c(H0[ind.haz]*expx_bet_w)
+llpennlognGeni=(-sum(censor*(c(x_bet_w)))+sum(n_eve*log(svexp_bet_xo))+
+(1/(2*a*(1-roh^2)))*sum(W1^2+W2^2-2*roh*W1*W2))
+Dw<-(-(censor)+c(H_bet_xw))
+Dw[i1]<-Dw[i1]+(W1-roh*W2)*(1/(a*(1-roh^2)))
+Dw[i2]<-Dw[i2]+(W2-roh*W1)*(1/(a*(1-roh^2)))
+attr(llpennlognGeni,"gradient")<-c(c(-sumx+dbsvexp_bet_xo),c(Dw))
+llpennlognGeni
+}
+
+Hesfunc1<-function(par,X,RI,n_eve,ind.haz,i1,i2,i3){
+n4=ncol(X);n=nrow(X)
+bet=par[array(1:n4)]
+W=par[(1+n4):(length(par))]
+W1=W[i1];W2=W[i2]
+x_bet<-c(X%*%bet+W)
+expx_bet<-exp(c(x_bet))
+svexp_bet_xo=as.vector(RI%*%(exp(x_bet)))
+H0<-cumsum(c(n_eve/svexp_bet_xo))#
+as0=H0[ind.haz]
+ak=cumsum(c((n_eve/svexp_bet_xo^2)))
+as2=ak[ind.haz]
+as1=(as2*c(expx_bet^2)-(as0*c(expx_bet)))
+as3=as2[i3]*(expx_bet[i1])*(expx_bet[i2])
+xx<-matrix(c(W1,W2,-as1[i1],-as3,-as3,-as1[i2]),(n/2),6)
+xx
+}
+
+Hesfunc0<-function(par,X,RI,n_eve,ind.haz,i1,i2,i3){
+n4=ncol(X);n=nrow(X)
+bet=par[array(1:n4)]
+W=par[(1+n4):(length(par))]
+W1=W[i1];W2=W[i2]
+x_bet<-c(X%*%bet+W)
+expx_bet<-exp(c(x_bet))
+svexp_bet_xo=as.vector(RI%*%(exp(x_bet)))
+H0<-cumsum(c(n_eve/svexp_bet_xo))###obtain initial parameter for H0(t)
+as0=H0[ind.haz]
+ak=cumsum(c((n_eve/svexp_bet_xo^2)))
+as2=ak[ind.haz]
+as1=(as2*c(expx_bet^2)-(as0*c(expx_bet)))
+as3=as2[i3]*(expx_bet[i1])*(expx_bet[i2])
+xx<-matrix(c(W1,W2,as1[i1],as1[i2],as3),(n/2),5)
+xx
+}
+
+newmatxoppp <- function(yy,daDI,drDI,DId2a,DIdade,DId2e){
+WI=matrix(c(yy[1],yy[2],yy[3],yy[4]),2,2)
+HIda=WI%*%daDI
+HIde=WI%*%drDI
+c((0.5*(sum(diag(((t(HIda)%*%HIda)-(WI%*%DId2a)))))),
+(0.5*(sum(diag(((t(HIda)%*%HIde)-(WI%*%DIdade)))))),
+(0.5*(sum(diag(((t(HIde)%*%HIde)-(WI%*%DId2e)))))))
+}
+
+.Etsubila= function (thetaa,xx){
+xx=as.matrix(xx)
+a = abs(thetaa[1]);r = thetaa[2]
+n = nrow(xx)
+D=matrix(c(a,r*a,r*a,a),2,2);daD=matrix(c(1,r,r,1),2,2)
+drD=matrix(c(0,a,a,0),2,2);dadrD=matrix(c(0,1,1,0),2,2)
+DI=c(1/(a*(1-r^2)))*matrix(c(1,-r,-r,1),2,2)
+daDI=c(-1/(a^2*(1-r^2)))*matrix(c(1,-r,-r,1),2,2)
+drDI=c(1/(a*(1-r^2)^2))*matrix(c(2*r,-(1+r^2),-(1+r^2),2*r),2,2)
+DId2a=c(2/(a^3*(1-r^2)))*matrix(c(1,-r,-r,1),2,2)
+DId2e=c(1/(a*(1-r^2)^3))*matrix(c((2+6*r^2),-2*r*(3+r^2),-2*r*(3+r^2),(2+6*r^2)),2,2)
+DIdade=c(-1/(a^2*(1-r^2)^2))*matrix(c(2*r,-(1+r^2),-(1+r^2),2*r),2,2)
+part1=c(0.5*n*(sum(diag(((-daDI)%*%daD)))),0,0.5*n*(sum(diag(((-drDI)%*%drD)))))
+kr <- apply(xx,1,newmatxoppp,daDI=daDI,drDI=drDI,DId2a=DId2a,
+DIdade=DIdade,DId2e=DId2e)
+part=c(part1+colSums(t(kr)))
+hesi=matrix(c(part[1],part[2],part[2],part[3]),2,2)
+hesi}
+
+.sdLliklogncv = function (thetaa, xx){
+xx=as.matrix(xx)
+W1=xx[,1];W2=xx[,2];S1=xx[,3];S2=xx[,4];S3=xx[,5]
+a = abs(thetaa[1]) #var
+e = thetaa[2] #corr
+n=nrow(xx)
+P11<-(S1*S2-(1/(a*(1-e^2)))*(S1+S2)+
+(1/(a^2*(1-e^2)^2))-S3^2-2*(e/(a*(1-e^2)))*S3-(e^2/(a^2*(1-e^2)^2)))
+daP11<-( (1/(a^2*(1-e^2)))*(S1+S2)-(2/(a^3*(1-e^2)^2))+2*(e/(a^2*(1-e^2)))*S3+
+(2*e^2/(a^3*(1-e^2)^2)))
+deP11<- ( -(2*a*e/(a*(1-e^2))^2)*(S1+S2)+((4*a^2*(1-e^2)*e)/(a^2*(1-e^2)^2)^2)+
+2*(-1/(a*(1-e^2))-2*(e^2*a)/(a*(1-e^2))^2)*S3-
+(e*2/(a^2*(1-e^2)^2)+(4*a^2*(1-e^2)*e^3)/(a^2*(1-e^2)^2)^2))
+d2aP11<-( -(2/(a^3*(1-e^2)))*(S1+S2)+(6/(a^4*(1-e^2)^2))-4*(e/(a^3*(1-e^2)))*S3-
+(6*e^2/(a^4*(1-e^2)^2)))
+daRP11<-( ((2*a^2*e)/(a^2*(1-e^2))^2)*(S1+S2)-(8*(a^3*(1-e^2)*e)/(a^3*(1-e^2)^2)^2)+
+2*(1/(a^2*(1-e^2))+2*(a^2*e^2)/(a^2*(1-e^2))^2)*S3+
+(4*e/(a^3*(1-e^2)^2)+8*(a^3*(1-e^2)*e^3)/(a^3*(1-e^2)^2)^2))
+deP11<- ( -(2*a*e/(a*(1-e^2))^2)*(S1+S2)+((4*a^2*(1-e^2)*e)/(a^2*(1-e^2)^2)^2)+
+2*(-1/(a*(1-e^2))-2*(e^2*a)/(a*(1-e^2))^2)*S3-
+( e*2/(a^2*(1-e^2)^2)+(4*a^2*(1-e^2)*e^3)/(a^2*(1-e^2)^2)^2))
+d2eP11<- ( -(2*a/(a*(1-e^2))^2 + 8*a^2*e^2*(a*(1-e^2))/(a*(1-e^2))^4)*(S1+S2)+
+((4 * a^2 * (1 - e^2) - 4 * a^2 * (2 * e) * e)/(a^2 * (1 - e^2)^2)^2 +
+(4 * a^2 * (1 - e^2) * e) * (2 * (a^2 * (2 * (2 * e * (1 - e^2))) * (a^2 * (1 - e^2)^2)))/((a^2 * (1 - e^2)^2)^2)^2) +
+(-(2 * (a * (2 * e)/(a * (1 - e^2))^2 + (2 * (2 * e * a)/(a * (1 - e^2))^2 + 2 * (e^2 * a) * (2 * (a * (2 * e) * (a * (1 -
+    e^2))))/((a * (1 - e^2))^2)^2))))*S3-
+(2/(a^2 * (1 - e^2)^2) + e * 2 * (a^2 * (2 * (2 * e * (1 - e^2))))/(a^2 *
+    (1 - e^2)^2)^2 + ((4 * a^2 * (1 - e^2) * (3 * e^2) - 4 *
+    a^2 * (2 * e) * e^3)/(a^2 * (1 - e^2)^2)^2 + (4 * a^2 * (1 -
+    e^2) * e^3) * (2 * (a^2 * (2 * (2 * e * (1 - e^2))) * (a^2 *
+    (1 - e^2)^2)))/((a^2 * (1 - e^2)^2)^2)^2)))
+D2a=((-(2 * (2 * a) * (1 - e^2)/(2 * a^2 * (1 - e^2))^2))*sum((W1^2+W2^2-2*e*W1*W2))+
+n/a^2-(1/2)*sum(d2aP11/P11-daP11^2/P11^2))
+DaR=((2 * a^2 * (2 * e)/(2 * a^2 * (1 - e^2))^2)*sum((W1^2+W2^2-2*e*W1*W2))-
+(1/(a^2*(1-e^2)))*sum(W1*W2)-(1/2)*sum(daRP11/P11-(daP11*deP11)/P11^2))
+D2R=( (-(4 * a/(2 * a * (1 - e^2))^2 + (4 * a * e) * (2 * (2 * a * (2 *
+    e) * (2 * a * (1 - e^2))))/((2 * a * (1 - e^2))^2)^2))*sum((W1^2+W2^2-2*e*W1*W2))+
+((2*a*e)/(a*(1-e^2))^2)*sum(W1*W2)+(a * (2 * e)/(a * (1 - e^2))^2)*sum(W1*W2)+
+n*( 1/(1-e^2)+2*e^2/(1-e^2)^2)-(1/2)*sum(d2eP11/P11-deP11^2/P11^2))
+hesi=matrix(c(D2a,DaR,DaR,D2R),2,2)
+-hesi
+}
+#
+
+SE.PPL= function(par,bet,newtht,X,RI,INC,n_eve,ind.haz,indx,Xoxo){
+ncoef=ncol(X);n=nrow(X)
+a=newtht[1];roh=newtht[2]
+i1=INC[,1];i2=INC[,2];i3=INC[,3]
+W=par
+x_bet_w<-X%*%bet+W
+expx_bet<-exp(c(x_bet_w))
+svexp_bet_xo=as.vector(RI%*%(exp(x_bet_w)))
+dbpart1svexp=(RI%*%(c(exp(x_bet_w))*X))
+af=c(n_eve/svexp_bet_xo)
+H0<-cumsum(c(n_eve/svexp_bet_xo))
+as0=H_bet_xw=H0[ind.haz]*expx_bet
+as0=as0[indx]
+vexpx_bet=expx_bet[indx]
+ak=cumsum(c((-n_eve/svexp_bet_xo^2)))
+as2=ak[ind.haz]*expx_bet
+as2=as2[indx]
+ma0<-matrix(0,n,n)
+ma0[lower.tri(ma0, diag = TRUE)]<-1
+ma1<-t((ma0*vexpx_bet))
+ma2<-ma1*c(as2)
+digg<-diag(ma2)+as0
+ma3<-t(ma2)
+D2W<-ma3+ma2
+diag(D2W)<-digg
+ma0[indx,]=D2W
+ma1=t(ma0)
+D2W[indx,]=ma1
+ma0<-ma1<-ma2<-ma3<-NULL
+diag(D2W)<-diag(D2W)+(1/(a*(1-roh^2)))
+D2W[cbind(i1,i2)]<-D2W[cbind(i1,i2)]-(roh/(a*(1-roh^2)))
+D2W[cbind(i2,i1)]<-D2W[cbind(i2,i1)]-(roh/(a*(1-roh^2)))
+dwbpart20<-(dbpart1svexp)*c(-n_eve/svexp_bet_xo^2)
+dwbpart21<-apply(as.matrix(dwbpart20),2,matcolcumsum)
+dwbpart22=dwbpart21[ind.haz,]
+DBW=dwbpart22*expx_bet+H_bet_xw*X
+dwbpart30<-(af*dbpart1svexp)
+dwbpart31<-(c(1/svexp_bet_xo)*dbpart1svexp)
+d2bpart1svexp=(RI%*%(c(expx_bet)*Xoxo))
+d2bsvexp_bet_xo=matrix(c(as.vector(colSums(af*d2bpart1svexp))),ncoef,ncoef)
+D2B=d2bsvexp_bet_xo-t(dwbpart30)%*%dwbpart31
+MA=rbind(D2B,DBW);MB=rbind(t(DBW),D2W)
+HES=cbind(MA,MB)
+AB<-try(chol(HES),silent=TRUE)
+if(length(attr(AB,"class"))==0){INVE<-chol2inv(AB)}
+if(length(attr(AB,"class"))!=0){INVE<-solve(HES)}
+se=sqrt(diag(INVE))
+SEofbetandsig=se[1:ncoef]
+er1=array(1:ncoef)
+inx=cbind(rep(er1,ncoef),rep(er1,each=ncoef))
+vcovb=matrix(c(INVE[inx]),ncoef,ncoef)
+ABC<-try(chol(D2W),silent=TRUE)
+if(length(attr(ABC,"class"))==0){INV<-chol2inv(ABC)}
+if(length(attr(ABC,"class"))!=0){INV<-solve(D2W)}
+ij=cbind(i1,i2);as3=INV[ij];as2=diag(INV)
+xx=matrix(c(as2[i1],as3,as3,as2[i2]),n/2,4)
+list(SE=SEofbetandsig,vcovb=vcovb,xx=xx)
+}
+
+Lliklogn= function (thetaa,xx,QQ){
+xx=as.matrix(xx)
+a = abs(thetaa[1]);r = thetaa[2]
+n = nrow(xx)
+xx[,3]=xx[,3]+(1/(a*(1-r^2)));xx[,6]=xx[,6]+(1/(a*(1-r^2)))
+xx[,4]=xx[,4]-(r/(a*(1-r^2)));xx[,5]=xx[,5]-(r/(a*(1-r^2)))
+W1=xx[,1];W2=xx[,2];s1=xx[,3];s2=xx[,6];s3=xx[,4]
+logli=(QQ-n*log(a)-0.5*n*log(1-r^2)-0.5*sum(log(s1*s2-s3^2)))
+logli}
+
+initfppar<- function(W,bet,tht,censor,X,RI,n_eve,ind.haz,indx,INC){
+newrhotry=c(array(-5:9)*0.1);n4=length(bet);n1=length(W);n=n1/2
+i1=INC[,1];i2=INC[,2];i3=INC[,3]
+llall0=1;paro=c(bet,W)
+for(q in 1:length(newrhotry)){
+fittrp=do.call(nlm, args=c(list(f=llpennlognGeni,p=paro,newtht=c(tht,newrhotry[q]),
+censor=censor,X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,steptol= 1e-04,fscale = 1,
+print.level = 0, ndigit = 12,iterlim= 100,gradtol = 1e-05,check.analyticals = FALSE)))
+paro=fittrp$estimate
+xx=Hesfunc1(par=paro,X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,i3=i3)
+QQ=-fittrp$minimum
+llall0[q]=Lliklogn(c(tht,newrhotry[q]),xx=xx,QQ=QQ)}
+ordr=sort(llall0, decreasing = TRUE, index.return = TRUE)
+indxl0=ordr$ix[1];rho<-newrhotry[indxl0];newtht=c(tht,rho)
+iter1=0
+repeat{
+iter1=iter1+1
+fittrp=do.call(nlm, args=c(list(f=llpennlognGeni,p=paro,newtht=newtht,
+censor=censor,X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,steptol= 1e-05,fscale = 1,
+print.level = 0, ndigit = 12,iterlim=20,gradtol = 1e-06,check.analyticals = FALSE)))
+paro=fittrp$estimate;W=paro[(1+n4):(n1+n4)];bet=paro[1:n4];W1=W[i1];W2=W[i2]
+oppD2W<-Inv.HesPPL(W=W,bet=bet,newtht=newtht,X=X,RI=RI,INC=INC,
+n_eve=n_eve,ind.haz=ind.haz,indx=indx)
+Hii=oppD2W$Hii;Hij=oppD2W$Hij;ddet=oppD2W$ddet
+wsq=sum(W1^2+W2^2);w12=2*sum(W1*W2)
+r0=(Hij+w12)/(Hii+wsq);a0=((Hii+wsq)-r0*(Hij+w12))/(2*n*(1-r0^2))
+if(a0<=0.00001){a0<-0.00001};if(r0<=c(-0.999)){r0<-c(-0.999)};if(r0>=c(0.999)){r0<-c(0.999)}
+newtht=c(a0,r0)
+lik=-n*log(a0)-0.5*n*log(1-r0^2)-0.5*ddet-fittrp$minimum
+if(iter1 >=3) break}
+list(newtht=newtht,par=paro,lik=lik)}
+
+initfpartht<- function(W,bet,tht,censor,X,RI,n_eve,ind.haz,indx,INC){
+newrhotry0=c(array(1:20)*0.05);n4=length(bet);n1=length(W);n=n1/2
+newrhotry=sort(c(newrhotry0,1.3,1.6,1.9,tht[1]))
+i1=INC[,1];i2=INC[,2];i3=INC[,3]
+llall0=1;paro=c(bet,W)
+for(q in 1:length(newrhotry)){
+fittrp=do.call(nlm, args=c(list(f=llpennlognGeni,p=paro,newtht=c(newrhotry[q],tht[2]),
+censor=censor,X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,steptol= 1e-04,fscale = 1,
+print.level = 0, ndigit = 12,iterlim= 100,gradtol = 1e-05,check.analyticals = FALSE)))
+paro=fittrp$estimate
+xx=Hesfunc1(par=paro,X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,i3=i3)
+QQ=-fittrp$minimum
+llall0[q]=Lliklogn(c(newrhotry[q],tht[2]),xx=xx,QQ=QQ)}
+ordr=sort(llall0, decreasing = TRUE, index.return = TRUE)
+indxl0=ordr$ix[1]
+newtht=c(newrhotry[indxl0],tht[2])
+iter1=0
+repeat{
+iter1=iter1+1
+fittrp=do.call(nlm, args=c(list(f=llpennlognGeni,p=paro,newtht=newtht,
+censor=censor,X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,steptol= 1e-05,fscale = 1,
+print.level = 0, ndigit = 12,iterlim=20,gradtol = 1e-06,check.analyticals = FALSE)))
+paro=fittrp$estimate;W=paro[(1+n4):(n1+n4)];bet=paro[1:n4];W1=W[i1];W2=W[i2]
+oppD2W<-Inv.HesPPL(W=W,bet=bet,newtht=newtht,X=X,RI=RI,INC=INC,
+n_eve=n_eve,ind.haz=ind.haz,indx=indx)
+Hii=oppD2W$Hii;Hij=oppD2W$Hij;ddet=oppD2W$ddet
+wsq=sum(W1^2+W2^2);w12=2*sum(W1*W2)
+r0=(Hij+w12)/(Hii+wsq);a0=((Hii+wsq)-r0*(Hij+w12))/(2*n*(1-r0^2))
+if(a0<=0.00001){a0<-0.00001};if(r0<=c(-0.999)){r0<-c(-0.999)};if(r0>=c(0.999)){r0<-c(0.999)}
+newtht=c(a0,r0)
+lik=-n*log(a0)-0.5*n*log(1-r0^2)-0.5*ddet-fittrp$minimum
+if(iter1 >=3) break}
+list(newtht=newtht,par=paro,lik=lik)}
+
+
+Inv.HesPPL<-function(W,bet,newtht,X,RI,INC,n_eve,ind.haz,indx){
+ncoef=ncol(X);n=nrow(X)
+i1=INC[,1];i2=INC[,2];i3=INC[,3]
+x_bet_w<-X%*%bet+W
+expx_bet<-exp(c(x_bet_w))
+svexp_bet_xo=as.vector(RI%*%(exp(x_bet_w)))
+H0<-cumsum(c(n_eve/svexp_bet_xo))
+as0=H_bet_xw=H0[ind.haz]*expx_bet
+as0=as0[indx]
+vexpx_bet=expx_bet[indx]
+ak=cumsum(c((-n_eve/svexp_bet_xo^2)))
+as2=ak[ind.haz]*expx_bet
+as2=as2[indx]
+ma0<-matrix(0,n,n)
+ma0[lower.tri(ma0, diag = TRUE)]<-1
+ma1<-t((ma0*vexpx_bet))
+ma2<-ma1*c(as2)
+digg<-diag(ma2)+as0
+ma3<-t(ma2)
+D2W<-ma3+ma2
+diag(D2W)<-digg
+ma0[indx,]=D2W
+ma1=t(ma0)
+D2W[indx,]=ma1
+ma0<-ma1<-ma2<-ma3<-NULL
+a=newtht[1];roh=newtht[2]
+diag(D2W)<-diag(D2W)+(1/(a*(1-roh^2)))
+D2W[cbind(i1,i2)]<-D2W[cbind(i1,i2)]-(roh/(a*(1-roh^2)))
+D2W[cbind(i2,i1)]<-D2W[cbind(i2,i1)]-(roh/(a*(1-roh^2)))
+AB<-try(chol(D2W),silent=TRUE)
+if(length(attr(AB,"class"))==0){INVE<-chol2inv(AB);diet<-2*sum(log(diag(AB)))}
+if(length(attr(AB,"class"))!=0){INVE<-solve(D2W);diet<-log(det(D2W))}
+ij=cbind(i1,i2);as3=INVE[ij];ij=cbind(i2,i1);as4=INVE[ij];as2=diag(INVE)
+Hii=sum(as2);Hij=sum(as3+as4)
+if(is.infinite(diet)){
+ij=cbind(i1,i2)
+as3=D2W[ij];as2=diag(D2W)
+diet=sum(log(as2[i1]*as2[i2]-as3^2))}
+list(Hii=Hii,Hij=Hij,ddet=diet)
+}
+
+
+sefr<-function(newtht0,yy,xx){
+haser=.Etsubila(thetaa=newtht0,xx=yy);INV=solve(haser)
+haser1=.sdLliklogncv(thetaa=newtht0,xx=xx)
+INV1=solve(haser1);setht=c(0,0);sethto=0
+if(INV[1,1]>0){setht[1]=sqrt(INV[1,1])}else{
+if(haser[1,1]>0){setht[1]=sqrt(c(1/haser[1,1]))}else{setht[1]=sqrt(INV1[1,1])}}
+if(INV[2,2]>0){sethto=sqrt(INV[2,2])
+if(sethto>2){setht[2]=sqrt(INV1[2,2])}else{setht[2]=sethto}}else{setht[2]=sqrt(INV1[2,2])}
+setht}
+
+initial<-function(initfrailp,bet,time,censor,X,RI,n_eve,ind.haz,i1,i2,indx,INC,tracce){
+data.n1=length(time);data.n=data.n1/2;ncovar_coef=length(bet)
+if(length(initfrailp)>0){newtht<-initfrailp;W=rep(0,data.n1)
+tracce[1,]<-c(bet,newtht,1)
+fittrp=do.call(nlm, args=c(list(f=llpennlognGeni,p=c(bet,W),newtht=newtht,
+censor=censor,X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,steptol= 1e-04,fscale = 1,
+print.level = 0, ndigit = 12,iterlim=20,gradtol = 1e-05,check.analyticals = FALSE)))
+paro=fittrp$estimate
+W=paro[(1+ncovar_coef):(data.n1+ncovar_coef)];bet=paro[1:ncovar_coef]
+prilim<-initfpartht(W=W,bet=bet,tht=initfrailp,censor=censor,X=X,RI=RI,n_eve=n_eve,
+ind.haz=ind.haz,indx=indx,INC=INC)
+newtht=prilim$newtht;paro=prilim$par;lik=prilim$lik;tracce[2,]<-c(bet,newtht,1)}else{
+datu=data.frame(time=time,censor=censor,X)
+IID=array(1:data.n1);ar=array(1:data.n);PID=rep(ar,each=2)
+form<-as.formula(paste(paste("Surv(time,censor)"),"~",paste(colnames(X), collapse = "+ "),
+"+ frailty.gaussian(IID)",collapse=""))
+cph.fit <- coxph(form,data=datu,method = "breslow")
+bet=cph.fit$coefficients
+re=cph.fit$history$`frailty.gaussian(IID)`$history
+re1=re[,1]
+tht0=re1[length(re1)]
+if(tht0<0.01){
+IID=array(1:data.n1);ar=array(1:data.n);PID=rep(ar,each=2)
+form<-as.formula(paste(paste("Surv(time,censor)"),"~",paste(colnames(X), collapse = "+ "),
+"+ frailty.gaussian(PID)",collapse=""))
+cph.fit <- coxph(form,data=datu,method = "breslow")
+bet=cph.fit$coefficients
+re=cph.fit$history$`frailty.gaussian(PID)`$history
+re1=re[,1]
+tht0=re1[length(re1)]}
+tracce[1,]<-c(bet,c(tht0,0),1)
+W=rep(0,data.n1)
+if(tht0>0.01){
+fittrp=do.call(nlm, args=c(list(f=llpennlognGeni,p=c(bet,W),newtht=c(tht0,0),
+censor=censor,X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,steptol= 1e-04,fscale = 1,
+print.level = 0, ndigit = 12,iterlim=20,gradtol = 1e-05,check.analyticals = FALSE)))
+paro=fittrp$estimate
+W=paro[(1+ncovar_coef):(data.n1+ncovar_coef)];bet=paro[1:ncovar_coef]
+prilim<-initfppar(W=W,bet=bet,tht=tht0,censor=censor,X=X,RI=RI,n_eve=n_eve,
+ind.haz=ind.haz,indx=indx,INC=INC)
+newtht=prilim$newtht;paro=prilim$par;lik=prilim$lik;tracce[2,]<-c(bet,newtht,1)}else{
+newtht<-c(0.01,0);paro=rep(0,(data.n1+length(bet)))
+paro[1:length(bet)]=bet;lik=cph.fit$loglik[2]*10
+tracce[2,]<-c(bet,newtht,1)}}
+list(paro=paro,newtht=newtht,tracce=tracce,lik=lik)
+}
+#
+###################################################################
+############### End Utilities for Bivariate Log normal frailty##########
+###################################################################
+#
+########### Functions for shrgamsp, Univariate and bivariate shared gamma############
+##############################################################################
 .SEunivgam=function(bet,newtht,n_eve,uniq_tim,H,h0,censor,time,X){
 a<-as.vector(newtht);n_eve<-as.vector(n_eve);uniq_tim<-as.vector(uniq_tim)
 h0<-as.vector(h0);time<-as.vector(time);HH=H;HH<-as.vector(HH)
@@ -925,6 +1377,8 @@ seeho=c(seofthet[(n_cov_coef+1):(length(seofthet)-1)])
 seho=cbind(unqtime=trevntime,ho=nonzero_h0,SEho=seeho)
 list(se=SEofbetandsig,vco=D2b0,seho=seho)
 }
+
+
 .SEunivgamw=function(bet,newtht,n_eve,etime,H,h0,censor,time,X,WE){
 a<-as.vector(newtht);n_eve<-as.vector(n_eve);etime<-as.vector(etime)
 h0<-as.vector(h0);time<-as.vector(time);HH=H;HH<-as.vector(HH);WE=c(WE)
@@ -1042,6 +1496,8 @@ seeho=c(seofthet[(n_cov_coef+1):(length(seofthet)-1)])
 seho=cbind(unqtime=trevntime,ho=nonzero_h0,SEho=seeho)
 list(se=SEofbetandsig,vco=D2b0,seho=seho)
 }
+
+
 .SEsharedgamw=function(bet,newtht,n_eve,etime,H,h0,censor,time,X,WE){
 a<-as.vector(newtht);n_eve<-as.vector(n_eve);etime<-as.vector(etime)
 h0<-as.vector(h0);time<-as.vector(time);HH=H;HH<-as.vector(HH);WE=WE
@@ -1317,220 +1773,10 @@ res$call <- match.call()
 class(res) <- c("shrgamsp")
 res
 }
-########End Univariate and Shared gamma############
-#######################################
-#
-###################################################################
-###############Utilities for Bivariate Log normal frailty##########
-###################################################################
-llpennlognGeni<-function (par,newtht,censor,X,RI,n_eve,ind.haz,i1,i2){
-ncoef=ncol(X);n1=nrow(X)
-a=newtht[1];roh=newtht[2]
-bet=par[1:ncoef]
-W=par[(ncoef+1):length(par)]
-W1=W[i1];W2=W[i2]
-x_bet<-X%*%bet
-x_bet_w<-X%*%bet+W
-expx_bet_w<-exp(c(x_bet_w))
-svexp_bet_xo=as.vector(RI%*%(exp(x_bet_w)))
-dbpart1svexp=(RI%*%(c(exp(x_bet_w))*X))
-af=c(n_eve/svexp_bet_xo)
-dbsvexp_bet_xo=as.vector(colSums(af*dbpart1svexp))
-sumx=as.vector(colSums((c(censor)*X)))
-H0<-cumsum(c(n_eve/svexp_bet_xo))
-H_bet_xw=c(H0[ind.haz]*expx_bet_w)
-llpennlognGeni=(-sum(censor*(c(x_bet_w)))+sum(n_eve*log(svexp_bet_xo))+
-(1/(2*a*(1-roh^2)))*sum(W1^2+W2^2-2*roh*W1*W2))
-Dw<-(-(censor)+c(H_bet_xw))
-Dw[i1]<-Dw[i1]+(W1-roh*W2)*(1/(a*(1-roh^2)))
-Dw[i2]<-Dw[i2]+(W2-roh*W1)*(1/(a*(1-roh^2)))
-attr(llpennlognGeni,"gradient")<-c(c(-sumx+dbsvexp_bet_xo),c(Dw))
-llpennlognGeni
-}
-
-Hesfunc0<-function(par,X,RI,n_eve,ind.haz,i1,i2,i3){
-n4=ncol(X);n=nrow(X);bet=par[array(1:n4)]
-W=par[(1+n4):(length(par))]
-W1=W[i1];W2=W[i2]
-x_bet<-c(X%*%bet+W)
-expx_bet<-exp(c(x_bet))
-svexp_bet_xo=as.vector(RI%*%(exp(x_bet)))
-H0<-cumsum(c(n_eve/svexp_bet_xo));as0=H0[ind.haz]
-ak=cumsum(c((n_eve/svexp_bet_xo^2)));as2=ak[ind.haz]
-as1=(as2*c(expx_bet^2)-(as0*c(expx_bet)))
-as3=as2[i3]*(expx_bet[i1])*(expx_bet[i2])
-xx<-matrix(c(W1,W2,as1[i1],as1[i2],as3),(n/2),5)
-xx
-}
-
-.sdLliklogncv = function (thetaa, xx){
-xx=as.matrix(xx)
-W1=xx[,1];W2=xx[,2];S1=xx[,3];S2=xx[,4];S3=xx[,5]
-a = abs(thetaa[1]) #var
-e = thetaa[2] #corr
-n=nrow(xx)
-P11<-(S1*S2-(1/(a*(1-e^2)))*(S1+S2)+
-(1/(a^2*(1-e^2)^2))-S3^2-2*(e/(a*(1-e^2)))*S3-(e^2/(a^2*(1-e^2)^2)))
-daP11<-( (1/(a^2*(1-e^2)))*(S1+S2)-(2/(a^3*(1-e^2)^2))+2*(e/(a^2*(1-e^2)))*S3+
-(2*e^2/(a^3*(1-e^2)^2)))
-deP11<- ( -(2*a*e/(a*(1-e^2))^2)*(S1+S2)+((4*a^2*(1-e^2)*e)/(a^2*(1-e^2)^2)^2)+
-2*(-1/(a*(1-e^2))-2*(e^2*a)/(a*(1-e^2))^2)*S3-
-(e*2/(a^2*(1-e^2)^2)+(4*a^2*(1-e^2)*e^3)/(a^2*(1-e^2)^2)^2))
-d2aP11<-( -(2/(a^3*(1-e^2)))*(S1+S2)+(6/(a^4*(1-e^2)^2))-4*(e/(a^3*(1-e^2)))*S3-
-(6*e^2/(a^4*(1-e^2)^2)))
-daRP11<-( ((2*a^2*e)/(a^2*(1-e^2))^2)*(S1+S2)-(8*(a^3*(1-e^2)*e)/(a^3*(1-e^2)^2)^2)+
-2*(1/(a^2*(1-e^2))+2*(a^2*e^2)/(a^2*(1-e^2))^2)*S3+
-(4*e/(a^3*(1-e^2)^2)+8*(a^3*(1-e^2)*e^3)/(a^3*(1-e^2)^2)^2))
-deP11<- ( -(2*a*e/(a*(1-e^2))^2)*(S1+S2)+((4*a^2*(1-e^2)*e)/(a^2*(1-e^2)^2)^2)+
-2*(-1/(a*(1-e^2))-2*(e^2*a)/(a*(1-e^2))^2)*S3-
-( e*2/(a^2*(1-e^2)^2)+(4*a^2*(1-e^2)*e^3)/(a^2*(1-e^2)^2)^2))
-d2eP11<- ( -(2*a/(a*(1-e^2))^2 + 8*a^2*e^2*(a*(1-e^2))/(a*(1-e^2))^4)*(S1+S2)+
-((4 * a^2 * (1 - e^2) - 4 * a^2 * (2 * e) * e)/(a^2 * (1 - e^2)^2)^2 +
-(4 * a^2 * (1 - e^2) * e) * (2 * (a^2 * (2 * (2 * e * (1 - e^2))) * (a^2 * (1 - e^2)^2)))/((a^2 * (1 - e^2)^2)^2)^2) +
-(-(2 * (a * (2 * e)/(a * (1 - e^2))^2 + (2 * (2 * e * a)/(a * (1 - e^2))^2 + 2 * (e^2 * a) * (2 * (a * (2 * e) * (a * (1 -
-    e^2))))/((a * (1 - e^2))^2)^2))))*S3-
-(2/(a^2 * (1 - e^2)^2) + e * 2 * (a^2 * (2 * (2 * e * (1 - e^2))))/(a^2 *
-    (1 - e^2)^2)^2 + ((4 * a^2 * (1 - e^2) * (3 * e^2) - 4 *
-    a^2 * (2 * e) * e^3)/(a^2 * (1 - e^2)^2)^2 + (4 * a^2 * (1 -
-    e^2) * e^3) * (2 * (a^2 * (2 * (2 * e * (1 - e^2))) * (a^2 *
-    (1 - e^2)^2)))/((a^2 * (1 - e^2)^2)^2)^2)))
-D2a=((-(2 * (2 * a) * (1 - e^2)/(2 * a^2 * (1 - e^2))^2))*sum((W1^2+W2^2-2*e*W1*W2))+
-n/a^2-(1/2)*sum(d2aP11/P11-daP11^2/P11^2))
-DaR=((2 * a^2 * (2 * e)/(2 * a^2 * (1 - e^2))^2)*sum((W1^2+W2^2-2*e*W1*W2))-
-(1/(a^2*(1-e^2)))*sum(W1*W2)-(1/2)*sum(daRP11/P11-(daP11*deP11)/P11^2))
-D2R=( (-(4 * a/(2 * a * (1 - e^2))^2 + (4 * a * e) * (2 * (2 * a * (2 *
-    e) * (2 * a * (1 - e^2))))/((2 * a * (1 - e^2))^2)^2))*sum((W1^2+W2^2-2*e*W1*W2))+
-((2*a*e)/(a*(1-e^2))^2)*sum(W1*W2)+(a * (2 * e)/(a * (1 - e^2))^2)*sum(W1*W2)+
-n*( 1/(1-e^2)+2*e^2/(1-e^2)^2)-(1/2)*sum(d2eP11/P11-deP11^2/P11^2))
-hesi=matrix(c(D2a,DaR,DaR,D2R),2,2)
--hesi
-}
-#
-#
-Inv.HesPPL<-function(W,bet,newtht,X,RI,INC,n_eve,ind.haz,indx){
-ncoef=ncol(X);n=nrow(X)
-i1=INC[,1];i2=INC[,2];i3=INC[,3]
-x_bet_w<-X%*%bet+W
-expx_bet<-exp(c(x_bet_w))
-svexp_bet_xo=as.vector(RI%*%(exp(x_bet_w)))
-H0<-cumsum(c(n_eve/svexp_bet_xo))
-as0=H_bet_xw=H0[ind.haz]*expx_bet
-as0=as0[indx]
-vexpx_bet=expx_bet[indx]
-ak=cumsum(c((-n_eve/svexp_bet_xo^2)))
-as2=ak[ind.haz]*expx_bet
-as2=as2[indx]
-ma0<-matrix(0,n,n)
-ma0[lower.tri(ma0, diag = TRUE)]<-1
-ma1<-t((ma0*vexpx_bet))
-ma2<-ma1*c(as2)
-digg<-diag(ma2)+as0
-ma3<-t(ma2)
-D2W<-ma3+ma2
-diag(D2W)<-digg
-ma0[indx,]=D2W
-ma1=t(ma0)
-D2W[indx,]=ma1
-ma0<-ma1<-ma2<-ma3<-NULL
-a=newtht[1];roh=newtht[2]
-diag(D2W)<-diag(D2W)+(1/(a*(1-roh^2)))
-D2W[cbind(i1,i2)]<-D2W[cbind(i1,i2)]-(roh/(a*(1-roh^2)))
-D2W[cbind(i2,i1)]<-D2W[cbind(i2,i1)]-(roh/(a*(1-roh^2)))
-AB<-try(chol(D2W),silent=TRUE)
-if(length(attr(AB,"class"))==0){INVE<-chol2inv(AB);diet<-2*sum(log(diag(AB)))}
-if(length(attr(AB,"class"))!=0){INVE<-solve(D2W);diet<-log(det(D2W))}
-ij=cbind(i1,i2);as3=INVE[ij];ij=cbind(i2,i1);as4=INVE[ij];as2=diag(INVE)
-Hii=sum(as2);Hij=sum(as3)
-if(is.infinite(diet)){
-ij=cbind(i1,i2)
-as3=D2W[ij];as2=diag(D2W)
-diet=sum(log(as2[i1]*as2[i2]-as3^2))}
-list(Hii=Hii,Hij=Hij,ddet=diet)
-}
-
-.Etsula= function (thetaa,xx){
-xx=as.matrix(xx);a = abs(thetaa[1]);r = thetaa[2];n = nrow(xx)
-daDI=c(-1/(a^2*(1-r^2)))*matrix(c(1,-r,-r,1),2,2)
-drDI=c(1/(a*(1-r^2)^2))*matrix(c(2*r,-(1+r^2),-(1+r^2),2*r),2,2)
-part1=c(0.5*(2*n/a^2-c(2/(a^3*(1-r^2)))*sum(xx[,1]+xx[,3]-2*r*xx[,2])),
-0.5*( (1/(a^2*(1-r^2)^2))*sum(2*r*(xx[,1]+xx[,3])-2*(1+r^2)*xx[,2])),
-0.5*((2*n*(1+r^2)/(1-r^2)^2)-c(1/(a*(1-r^2)^3))*sum((2+6*r^2)*(xx[,1]+xx[,3])-2*(6*r+2*r^3)*xx[,2])))
-kr <- apply(xx,1,newmoppp,daDI=daDI,drDI=drDI)
-part=c(part1+colSums(t(kr)))
-hesi=matrix(c(part[1],part[2],part[2],part[3]),2,2)
-hesi}
-
-SEbothlognph= function(W,bet,newtht,X,RI,INC,n_eve,ind.haz,indx,Xoxo){
-ncoef=length(bet);n1=length(W);n=n1/2
-a=newtht[1];r=newtht[2]
-i1=INC[,1];i2=INC[,2];i3=INC[,3]
-x_bet_w<-X%*%bet+W
-expx_bet<-exp(c(x_bet_w))
-svexp_bet_xo=as.vector(RI%*%(exp(x_bet_w)))
-dbpart1svexp=(RI%*%(c(exp(x_bet_w))*X))
-af=c(n_eve/svexp_bet_xo)
-H0<-cumsum(c(n_eve/svexp_bet_xo))
-as0=H_bet_xw=H0[ind.haz]*expx_bet
-as0=as0[indx]
-vexpx_bet=expx_bet[indx]
-ak=cumsum(c((-n_eve/svexp_bet_xo^2)))
-as2=ak[ind.haz]*expx_bet
-as2=as2[indx]
-ma0<-matrix(0,n1,n1)
-ma0[lower.tri(ma0, diag = TRUE)]<-1
-ma1<-t((ma0*vexpx_bet))
-ma2<-ma1*c(as2)
-digg<-diag(ma2)+as0
-ma3<-t(ma2)
-D2W<-ma3+ma2
-diag(D2W)<-digg
-ma0[indx,]=D2W
-ma1=t(ma0)
-D2W[indx,]=ma1
-ma0<-ma1<-ma2<-ma3<-NULL
-diag(D2W)<-diag(D2W)+(1/(a*(1-r^2)))
-D2W[cbind(i1,i2)]<-D2W[cbind(i1,i2)]-(r/(a*(1-r^2)))
-D2W[cbind(i2,i1)]<-D2W[cbind(i2,i1)]-(r/(a*(1-r^2)))
-dwbpart20<-(dbpart1svexp)*c(-n_eve/svexp_bet_xo^2)
-dwbpart21<-apply(as.matrix(dwbpart20),2,matcolcumsum)
-dwbpart22=dwbpart21[ind.haz,]
-DBW=dwbpart22*expx_bet+H_bet_xw*X
-dwbpart30<-(af*dbpart1svexp)
-dwbpart31<-(c(1/svexp_bet_xo)*dbpart1svexp)
-d2bpart1svexp=(RI%*%(c(expx_bet)*Xoxo))
-d2bsvexp_bet_xo=matrix(c(as.vector(colSums(af*d2bpart1svexp))),ncoef,ncoef)
-D2B=d2bsvexp_bet_xo-t(dwbpart30)%*%dwbpart31
-MA=rbind(D2B,DBW);MB=rbind(t(DBW),D2W)
-HES=cbind(MA,MB)
-AB<-try(chol(HES),silent=TRUE)
-if(length(attr(AB,"class"))==0){INVE<-chol2inv(AB)}
-if(length(attr(AB,"class"))!=0){INVE<-solve(HES)}
-se=sqrt(diag(INVE));SEofbetandsig=se[1:ncoef];er1=array(1:ncoef)
-inx=cbind(rep(er1,ncoef),rep(er1,each=ncoef))
-vcovb=matrix(c(INVE[inx]),ncoef,ncoef);HES<-INVE<-AB<-NULL
-ABC<-try(chol(D2W),silent=TRUE)
-if(length(attr(ABC,"class"))==0){INV<-chol2inv(ABC)}
-if(length(attr(ABC,"class"))!=0){INV<-solve(D2W)}
-ij=cbind(i1,i2);ji=cbind(i2,i1)
-as3=INV[ij];as2=diag(INV);xx0=matrix(c(as2[i1],as3,as2[i2]),n,3);ABC<-D2W<-INV<-NULL
-haser=.Etsula(thetaa=newtht,xx=xx0);INV=solve(haser);dh=diag(INV)
-if(any(dh<0)){xx=Hesfunc0(par=c(bet,W),X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,i3=i3)
-haser1=.sdLliklogncv(thetaa=newtht,xx=xx);INV1=solve(haser1);dh0=diag(INV1);setht=sqrt(dh0)
-if(dh[1]>0){setht[1]=sqrt(dh[1])}else{if(haser[1,1]>0){
-setht[1]=sqrt(c(1/haser[1,1]))}}}else{setht=sqrt(dh)}
-if(setht[2]>2){xx=Hesfunc0(par=c(bet,W),X=X,RI=RI,n_eve=n_eve,ind.haz=ind.haz,i1=i1,i2=i2,i3=i3)
-haser1=.sdLliklogncv(thetaa=newtht,xx=xx);INV1=solve(haser1)
-dh0=diag(INV1);setht0=sqrt(dh0);setht=c(max(setht0[1],setht[1]),setht0[2])}
-list(SE=SEofbetandsig,vcovb=vcovb,setht=setht)
-}
-#
-###################################################################
-############### End Utilities for Bivariate Log normal frailty##########
-#############################################################################
-#
+########### End  Functions for shrgamsp, Univariate and bivariate shared gamma############
+##############################################################################
 #
 ######### Function used To generate data from bcfrailph##########
-#################################################################
 genbcfrail<-function(psize,cenr,cent1,cent2,beta,frailty,fpar,bhaz,bhazpar,
 covartype,covarpar,inputcovar,covar1,covar2,comncovar){
 n<-psize; n1=n*2;IID=array(1:n1);PID=1;e1=array(1:n);indic2=2*e1;indic1=indic2-1
@@ -1617,4 +1863,8 @@ Y=matrix(c(time,censor),n1,2)
 list(dataa=dataa,X=RW,Y=Y)
 }
 #
-#########End generate data from bcfrailph##########
\ No newline at end of file
+#########End generate data from bcfrailph##########
+
+
+
+
